{*************************************************************************
*                                                                        *
* Funktion: Digitales FIR Filter nach der DGL                            *
*                                                                        *
* y[k] = x[k] b[0] + x[k-1] b[1] . . . + x[k-(N-1)]b[N-1]                *
*                                                                        *
* Eingangswert:  MX0 = Aktueller Eingangswert x[k]                       *
*                MY0 = N-1 (Anzahl Koeffizienten -1 als Schleifenzaehler)*
*                                                                        *
* Ausgangswert:  MR0 = Aktueller Ausgangswert y[k] falls Integer-Mode    *
*                MR1 = Aktueller Ausgangswert y[k] falls 1.15-Mode       *
*                                                                        *
* Parameter:     I4 -> zeigt auf b[0] des Koeffizientenvektors im PM     *
*                L4 = N; (Anzahl zirkulaere Koeffizienten)               *
*                I3 -> zeigt auf einen zirkulaeren Verzoegerungs-Puffer  *
*                      im DM mit N Werten                                *
*                L3 = N (WICHTIG)                                        *
*                M1 = 1, M2 = -1, M5 = 1                                 *
*                                                                        *
* Aufruf: CALL fir                                                       *
*                                                                        *
* Geaenderte Register:  MX0, MY0, MR, I3, I4,                            *
*                                                                        *
* J.Breidt, 11.05.06 Ver. 0.1
*************************************************************************}



.MODULE/RAM fir_fkt;
.ENTRY fir;
.ENTRY adap_fkt;


{-----------------------------------------------Funktionseinsprung-------  }

fir:
	ENA M_MODE;			{	Integer-Mode	}

	M5 = 1;
        { N-1 sichern	}
	AX0 = MY0;
	{	MAC init	}
	MR = 0	;
	{	Ersten Coef. laden	}
        MY0 = PM(I4,M5);

	{	ltesten Wert in den DelayBuffer schreiben	}
	DM(I3,M3) = MX0;

        {	CNTR = N-1	}
	CNTR = AX0;
	DO filter UNTIL CE;
filter:	MR = MR + MX0*MY0(uu),MX0 = DM (I3,M3), MY0 = PM(I4,M5);

	{	zuletzt geladene Werte verarbeiten	}
	MR = MR + MX0*MY0(uu);

        MODIFY(I3,M2);


RTS;
{ -------------------------------------------------Ende der Funktion   ----}



{*************************************************************************
*                                                                        *
* Funktion: Adaption der Filterkoeffizienten nach LMS                    *
*                                                                        *
*	Eingang:                                                         *
*		I4 -> g[0]  Filterkoeffizienten                          *
*		L4 = tabs                                                *
*                                                                        *
*		I3 -> x[k-(N-1)]  Verzgerungspuffer                     *
*		L3 = tabs                                                *
*                                                                        *
*		MX1 = step                                               *
*               MY1 = e[k] Fehler                                        *
*		MX0 = Aktueller Eingangswert x[k]                        *
*                                                                        *
*	Ausgabe:                                                         *
*		I7 -> g[0]  adaptierte Filterkoeffizienten               *
*                                                                        *
*                                                                        *
*	Modifikatoren:                                                   *
*		M0, M4                                                   *
*		M5 = 1                                                   *
*                                                                        *
* J.Breidt, 11.05.06 Ver. 0.1                                             *
*                                                                        *
*************************************************************************}

{-----------------------------------------------Funktionseinsprung-------  }

adap_fkt:
	ENA M_MODE;			{	Integer-Mode	}

	M5 = 1;
	M4 = 0;
	M2 = -1;
	M6 = -1;

        { Tabs sichern	}
	AY0 = MY0;

	{	MAC init	}
	MR = 0;
        {  MR = e[k] * step     }
        MR = MX1 * MY1 (ss);          { e[k] * step             }
        MY0 = MR0;                    {MY0 = e[k] * step        }

        {	CNTR = Tabs-1	}
	CNTR = AY0;
	DO adap UNTIL CE;
	{x[k-i] * step * e[k]   }
	MR = MX0*MY0(ss),MX0 = DM (I3,M2);   {MR =  x[k-i] * e[k] * step , MX0 = x[k-i]}
	{AR = w_old[k] + x[k-i] * step * e[k], AR => w_new[k+1]   }
	AX0 = MR0;
	AY0 = PM(I4,M5);
        AR = AX0 +AY0;

        {neuen coef speichern und I4+1       }
 adap:  PM(I7,M5) = AR;

	{	zuletzt geladene Werte verarbeiten	}
	MR =  MX0*MY0(ss);
	AX0 = MR0;
	AY0 = PM(I4,M5);
	AR = AX0 + AY0;
        {neuen coef speichern und I7+1       }
        PM(I7,M5) = AR;

        MODIFY(I3,M2);


RTS;

{ -------------------------------------------------Ende der Funktion   ----}

.ENDMOD;
